@@ -4,6 +4,8 @@ from pydantic import BaseModel, ConfigDict
 import plotly.graph_objects as go
 import pandas as pd
 from IPython.display import SVG, display
+from aryaxai.client.client import APIClient
+from aryaxai.common.xai_uris import EXPLAINABILITY_SUMMARY, GET_TRIGGERS_DAYS_URI
 
 
 class Case(BaseModel):
@@ -14,6 +16,8 @@ class Case(BaseModel):
     observations: List
     shap_feature_importance: Optional[Dict] = {}
     lime_feature_importance: Optional[Dict] = {}
+    ig_features_importance: Optional[Dict] = {}
+    dlb_feature_importance: Optional[Dict] = {}
     similar_cases: List
     is_automl_prediction: Optional[bool] = False
     model_name: str
@@ -27,10 +31,20 @@ class Case(BaseModel):
     created_at: Optional[str] = ""
     data: Optional[Dict] = {}
     similar_cases_data: Optional[List] = []
+    audit_trail: Optional[dict] = {}
+    project_name: Optional[str] = ""
     image_data: Optional[Dict] = {}
+    data_id: Optional[str] = ""
+    summary: Optional[str] = ""
 
     model_config = ConfigDict(protected_namespaces=())
 
+    api_client: APIClient
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.api_client = kwargs.get("api_client")
+
     def explainability_shap_feature_importance(self):
         """Plots Shap Feature Importance chart"""
         fig = go.Figure()
@@ -68,6 +82,43 @@ class Case(BaseModel):
         )
         fig.show(config={"displaylogo": False})
 
+    def explainability_ig_feature_importance(self):
+        """Plots IG Feature Importance chart"""
+        fig = go.Figure()
+
+        if len(list(self.ig_features_importance.values())) < 1:
+            return "No IG Feature Importance for the case"
+
+        if isinstance(list(self.ig_features_importance.values())[0], dict):
+            for col in self.ig_features_importance.keys():
+                fig.add_trace(
+                    go.Bar(
+                        x=list(self.ig_features_importance[col].values()),
+                        y=list(self.ig_features_importance[col].keys()),
+                        orientation="h",
+                        name=col,
+                    )
+                )
+        else:
+            fig.add_trace(
+                go.Bar(
+                    x=list(self.ig_features_importance.values()),
+                    y=list(self.ig_features_importance.keys()),
+                    orientation="h",
+                )
+            )
+        fig.update_layout(
+            barmode="relative",
+            height=800,
+            width=800,
+            yaxis_autorange="reversed",
+            bargap=0.01,
+            legend_orientation="h",
+            legend_x=0.1,
+            legend_y=1.1,
+        )
+        fig.show(config={"displaylogo": False})
+
     def explainability_lime_feature_importance(self):
         """Plots Lime Feature Importance chart"""
         fig = go.Figure()
@@ -105,6 +156,42 @@ class Case(BaseModel):
         )
         fig.show(config={"displaylogo": False})
 
+    def explainability_dlb_feature_importance(self):
+        """Plots DLB Feature Importance chart"""
+        fig = go.Figure()
+        if len(list(self.dlb_feature_importance.values())) < 1:
+            return "No DLB Feature Importance for the case"
+
+        if isinstance(list(self.dlb_feature_importance.values())[0], dict):
+            for col in self.dlb_feature_importance.keys():
+                fig.add_trace(
+                    go.Bar(
+                        x=list(self.dlb_feature_importance[col].values()),
+                        y=list(self.dlb_feature_importance[col].keys()),
+                        orientation="h",
+                        name=col,
+                    )
+                )
+        else:
+            fig.add_trace(
+                go.Bar(
+                    x=list(self.dlb_feature_importance.values()),
+                    y=list(self.dlb_feature_importance.keys()),
+                    orientation="h",
+                )
+            )
+        fig.update_layout(
+            barmode="relative",
+            height=800,
+            width=800,
+            yaxis_autorange="reversed",
+            bargap=0.01,
+            legend_orientation="h",
+            legend_x=0.1,
+            legend_y=1.1,
+        )
+        fig.show(config={"displaylogo": False})
+
     def explainability_prediction_path(self):
         """Explainability Prediction Path"""
         svg = SVG(self.case_prediction_svg)
@@ -366,3 +453,111 @@ class Case(BaseModel):
         )
 
         fig.show(config={"displaylogo": False})
+
+    def alerts_trail(self, page_num: Optional[int] = 1, days: Optional[int] = 7):
+        if days==7:
+            return pd.DataFrame(self.audit_trail.get("alerts", {}))
+        resp = self.api_client.post(f"{GET_TRIGGERS_DAYS_URI}?project_name={self.project_name}&page_num={page_num}&days={days}")
+        if resp.get("details"):
+            return pd.DataFrame(resp.get("details"))
+        else:
+            return "No alerts found."
+
+    def audit(self):
+        return self.audit_trail
+      
+    def feature_importance(self, feature: str):
+        if self.shap_feature_importance:
+            return self.shap_feature_importance.get(feature, {})
+        elif self.lime_feature_importance:
+            return self.lime_feature_importance.get(feature, {})
+        elif self.ig_features_importance:
+            return self.ig_features_importance.get(feature, {})
+        else:
+            return "No Feature Importance found for the case"
+        
+    def explainability_summary(self):
+        if self.data_id and not self.summary:
+            payload = {
+                "project_name": self.project_name,
+                "viewed_case_id": self.data_id,
+            }
+            res = self.api_client.post(EXPLAINABILITY_SUMMARY,payload)
+            if not res.get("success"):
+                raise Exception(res.get("details","Failed to summarize"))
+            
+            self.summary = res.get("details")
+            return res.get("details")
+        
+        return self.summary
+
+class CaseText(BaseModel):
+    model_name: str
+    status: str
+    prompt: str
+    output: str
+    explainabiblity: Optional[Dict] = {}
+
+    def prompt(self):
+        """Get prompt"""
+        return self.prompt
+    
+    def output(self):
+        """Get output"""
+        return self.output
+    
+
+    def explainability_raw_data(self) -> pd.DataFrame:
+        """Explainability Raw Data
+
+        :return: raw data dataframe
+        """
+        raw_data_df = (
+            pd.DataFrame([self.explainabiblity.get("feature_importance", {})])
+            .transpose()
+            .reset_index()
+            .rename(columns={"index": "Feature", 0: "Value"})
+            .sort_values(by="Value", ascending=False)
+        )
+        return raw_data_df
+    
+    def explainability_feature_importance(self):
+        """Plots Feature Importance chart"""
+        fig = go.Figure()
+        feature_importance = self.explainabiblity.get("feature_importance", {})
+
+        if not feature_importance:
+            return "No Feature Importance for the case"
+        raw_data_df = (
+            pd.DataFrame([feature_importance])
+            .transpose()
+            .reset_index()
+            .rename(columns={"index": "Feature", 0: "Value"})
+            .sort_values(by="Value", ascending=False)
+        )
+        fig.add_trace(
+            go.Bar(
+                x=raw_data_df["Value"],
+                y=raw_data_df["Feature"],
+                orientation="h"
+            )
+        )
+        fig.update_layout(
+            barmode="relative",
+            height=max(400, len(raw_data_df) * 20),
+            width=800,
+            yaxis=dict(
+                autorange="reversed",
+                tickmode="array",
+                tickvals=list(raw_data_df["Feature"]),
+                ticktext=list(raw_data_df["Feature"]),
+                tickfont=dict(size=10),
+            ),
+            bargap=0.01,
+            margin=dict(l=150, r=20, t=30, b=30),
+            legend_orientation="h",
+            legend_x=0.1,
+            legend_y=0.5,
+        )
+
+        fig.show(config={"displaylogo": False})
